#!/bin/bash
#
# mount-manager:bin/mount-manager.sh (Main entry point)
#
set -u
set -o pipefail

PROJECT_HOME="$( dirname "$( dirname "$( realpath "${BASH_SOURCE[0]}" )" )" )"
declare -r PROJECT_HOME

declare -r MOUNT_MANAGER_PATH="${PROJECT_HOME}/bin"
declare -r API_PATH="${MOUNT_MANAGER_PATH}/api"

declare -g  VERBOSE='false'
# shellcheck disable=SC2034
declare -gr REALUSER="${USER}"

# shellcheck disable=SC1090
source "${PROJECT_HOME}/apis/bash-common-api/bash-common-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/config-json-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/crypt-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/diskimage-config-json-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/iscsi-config-json-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/mount-manager-api.sh" || exit 1
# shellcheck disable=SC1090
source "${API_PATH}/samba-config-json-api.sh" || exit 1

function usage {
  cat <<EOF
Help to mount and umount samba and nfs shares, or diskimage and iscsi disks.

Usage:

$( basename "$0" ) --help
$( basename "$0" ) [--verbose]
$( basename "$0" ) --mount-all [--verbose]
$( basename "$0" ) --mount-default [--verbose]
$( basename "$0" ) --umount-all [--verbose]
$( basename "$0" ) --umount-default [--verbose]
$( basename "$0" ) --umount-all-force [--verbose]
$( basename "$0" ) --iscsi-logout [--verbose]

Actions:

  -h, --help            Display help and exit.
      --iscsi-logout    Logout all iSCSI connections and exit
  -M, --mount-all       Mount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true
  -U, --umount-all      Umount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true
  -D, --mount-default   Mount every thing define in configuration file with key:
                            "mount-default": true
  -X, --umount-default  Umount every thing define in configuration file with key:
                            "mount-default": true
  -F, --umount-all-force
                        Umount every thing define in configuration file, ignore
                        keys "mount-all" and "mount-default" contents.

Options:

  -v, --verbose         Show more information (debugging)

Related tools:

  * mount-manager-config
  * mount-manager-encrypt

EOF
}

function main {
  local function=
  local param=
  local param_all=
  local param_default=
  local param_verbose=

  while [ $# -gt 0 ] ; do
    param="$1"
    shift

    case "${param}" in
      '-h'|'--help')
        usage
        exit 0
        ;;

      '-M'|'--mount-all')
        function='mount_all'
        param_all='true'
        ;;

      '-D'|'--mount-default')
        function='mount_all'
        param_default='true'
        ;;

      '-U'|'--umount-all')
        function='umount_all'
        param_all='true'
        ;;

      '-F'|'--umount-all-force')
        function='umount_all'
        param_all='false'
        param_default='false'
        ;;

      '-X'|'--umount-default')
        function='umount_all'
        param_default='true'
        ;;

      '-v'|'--verbose')
        param_verbose='true'
        echo_verbose 'Verbose mode'
        ;;

      '--iscsi-logout')
        function='iscsi_logout'
        ;;

      *)
        echofatal "Unknown option: '${param}'"
        usage >&2
        exit 1
        ;;
    esac
  done

  export PARAM_ALL="${param_all}"
  export PARAM_DEFAULT="${param_default}"
  export VERBOSE="${param_verbose}"

  check_configuration

  init_CHECKLIST_values

  case "${function}" in
    'umount_all')
      umount_all
      ;;

    'mount_all')
      mount_all
      ;;

    'iscsi_logout')
      iscsi_logout
      ;;

    *)
      select_mountdirs
      ;;
  esac
}

main "$@"

